/***********************************************************************************************************
Copyright (c) 2012, Sebastian Breß, Otto-von-Guericke University of Magdeburg, Germany. All rights reserved.

This program and accompanying materials are made available under the terms of the 
GNU LESSER GENERAL PUBLIC LICENSE - Version 3, http://www.gnu.org/licenses/lgpl-3.0.txt
 **********************************************************************************************************/

/*!
 *  \file general_documentation.hpp    
 *  \brief This file contains additional documentation, like the generated web pages in the doxygen documentation.
 *  \author    Sebastian Breß
 *  \version   0.1
 *  \date      2012
 *  \copyright GNU LESSER GENERAL PUBLIC LICENSE - Version 3, http://www.gnu.org/licenses/lgpl-3.0.txt
 */


/*! \mainpage Documentation
 * \tableofcontents
 * \section intro_sec Introduction
 *
 * HyPE is a library build for automatic selection of processing units for coprocessing in database systems. The long-term goal of the project is to implement a fully fledged query processing engine, which is able to automatically generate and optimize a hybrid CPU/GPU physical query plan from a logical query plan.
 *	It is a research prototype developed by the <a href="http://www.uni-magdeburg.de/unimagdeburg/en/Otto_von_Guericke+University+Magdeburg.html">Otto-von-Guericke University Magdeburg</a> 
 * in collaboration with <a href="http://www.tu-ilmenau.de/en/international/">Ilmenau University of Technology</a>. 
 * 
 * See the publications listed below for more details.
 * 
 * \section features_sec Features
 * 
 * Currently, HyPE supports the following features:
 * - entirely written in C++
 * - easiliy extensible by utilizing plugin architecture
 * - decides on the (likely) optimal algorithm w.r.t. to a user specified optimization criterion for an operation
 * - requires (almost) no knwoledge about executed algorithms, just the relevant features of the datasets for an algorithms execution time
 * - collects statistical information to help the user to fine tune parameters of HyPE for their use case
 * 
 * 	\section sec_detailed_documentation Detailed Documentation
 * 
 *    -# \subpage page_install_hype 
 *    -# \subpage page_tutorial
 *    -# \subpage page_existing_plugins_hype
 *    -# \subpage page_configure_hype
 *    -# \subpage page_structure_hype
 *    -# \subpage page_extending_hype
 *    -# \subpage page_faq
 * 
 * \section members_sec Project Members
 * 
 * 
 * 
 * \subsection members_subsec Project Members:
 * - <a href="http://wwwiti.cs.uni-magdeburg.de/~bress/">Sebastian Breß</a> (University of Magdeburg)
 * - Klaus Baumann  (University of Magdeburg)
 * - Steven Ladewig (University of Magdeburg)
 * - Tobias Lauer (Jedox AG)
 * - Gunter Saake  (University of Magdeburg)
 * - Norbert Siegmund  (University of Magdeburg)
 * \subsection partners_subsec Project Partners:
 * - Felix Beier (Ilmenau University of Technology)
 * - Ladjel Bellatreche (LIAS/ISEA-ENSMA, Futuroscope, France)
 * - Hannes Rauhe (Ilmenau University of Technology)
 * - Kai-Uwe Sattler (Ilmenau University of Technology)
 * \subsection former_members_subsec Former Project Members:
 * - Ingolf Geist (University of Magdeburg)
 * 
 * 
 * 
 * \section publication_sec Publications 
\htmlonly
<ul class="bib2xhtml">

<li><a name="thesisBress"></a>Sebastian Bre&szlig;.
<a href="http://wwwiti.cs.uni-magdeburg.de/iti_db/publikationen/ps/auto/thesisBress.pdf">Ein selbstlernendes Entscheidungsmodell f&uuml;r die Verteilung von
  Datenbankoperationen auf CPU/GPU-Systemen</a>.
Master thesis, University of Magdeburg, Germany, March 2012.
In German.</li>

<!-- Authors: Sebastian Bress and Siba Mohammad and Eike Schallehn -->
<li><a name="bress:2012:GvD:decision_model"></a>Sebastian Bre&szlig;, Siba Mohammad, and
  Eike Schallehn.
<a href="http://wwwiti.cs.uni-magdeburg.de/iti_db/publikationen/ps/auto/bress:2012:GvDB:decision_model.pdf">Self-Tuning Distribution of DB-Operations on
  Hybrid CPU/GPU Platforms</a>.
In <i>Proceedings of the 24st Workshop Grundlagen von Datenbanken
  (GvD)</i>, pages 89&ndash;94. CEUR-WS, 2012.</li>

<!-- Authors: Siba Mohammad and Sebastian Bress and Eike Schallehn -->
<!-- <li><a name="siba:2012:GvD:cloud_dbms"></a>Siba Mohammad, Sebastian Bre&szlig;, and
  Eike Schallehn.
<a href="http://ceur-ws.org/Vol-850/paper_mohammad.pdf">Cloud Data Management:
  a Short Overview and Comparison of Current Approaches</a>.
In <i>Proceedings of the 24st Workshop Grundlagen von Datenbanken
  (GvD)</i>, pages 41&ndash;46. CEUR-WS, 2012.</li>-->

<!-- Authors: Sebastian Bress and Eike Schallehn and Ingolf Geist -->
<li><a name="bress:towards_hybrid_query_processing"></a>Sebastian Bre&szlig;, Eike Schallehn, and
  Ingolf Geist.
<a href="http://link.springer.com/chapter/10.1007/978-3-642-32518-2_3">Towards
  Optimization of Hybrid CPU/GPU Query Plans in Database Systems</a>.
In <i>Second ADBIS workshop on GPUs In Databases (GID)</i>, pages 27&ndash;35.
  Springer, 2012.</li>

<!-- Authors: Sebastian Bress and Felix Beier and Hannes Rauhe and Eike
  Schallehn and Kai Uwe Sattler and Gunter Saake -->
<li><a name="bress:adbis:2012"></a>Sebastian Bre&szlig;, Felix Beier, Hannes
  Rauhe, Eike Schallehn, Kai-Uwe Sattler, and Gunter Saake.
<a href="http://link.springer.com/chapter/10.1007/978-3-642-33074-2_5">Automatic Selection of Processing Units for Coprocessing in Databases</a>.
In <i>16th East-European Conference on Advances in Databases and Information
  Systems (ADBIS)</i>, pages 57&ndash;70. Springer, 2012.</li>

<!-- Authors: Sebastian Bress and Ingolf Geist and Eike Schallehn and Maik Mory
  and Gunter Saake -->
<li><a name="bress:journal:framework"></a>Sebastian Bre&szlig;, Ingolf Geist, Eike
  Schallehn, Maik Mory, and Gunter Saake.
A Framework for Cost based Optimization of Hybrid CPU/GPU Query Plans in
  Database Systems.
<cite>Control and Cybernetics</cite>, 41(4), 2012.
to appear.</li>

</ul>
\endhtmlonly
*/

/*! \page page_install_hype Installation
  \tableofcontents
  
   * 
	*   \section sec Installation on Ubuntu
  Just type the following commandline: <br>
  @code
  sudo apt-get install gcc g++ make doxygen doxygen-gui graphviz libboost-all-dev build-essential gnuplot-x11 ffmpeg libloki-dev libloki*
  @endcode
  Alternatively, you can use our installation script (Ubuntu):
  @code 
  ./setup-ubuntu.sh  
  @endcode


	* 
   * \section install_sec Installation
 * Currently only Linux is officially supported. For Installation, download and unpack the release package. Afterwards, you have to install the neccessary tools and libraries that the framework uses:

	- <a href="http://www.boost.org/">Boost</a>: boost_filesystem, boost_system, boost_thread, boost_program_options
	- <a href="http://loki-lib.sourceforge.net/">Loki Library</a>
	- <a href="http://www.alglib.net/">ALGLIB</a>
	 
	 We included a setup script for ubuntu users: <b>setup-ubuntu.sh</b>.
  
    \subsection subsec_building_hype  Building HyPE in a Terminal
    
Open a terminal and navigate to the directory were you unpacked HyPE.  
    
  Compile HyPE library
  @code
  cd hype-library/
  bash setup_ubuntu.sh	#if you have not installed the required tools and you use ubuntu, you can use this bash script, otherwise you can skip it 
  make
  make documentation
  @endcode
  
  Compile example program:  
  @code
  cd examples/use_as_online_framework/
  make
  make run  #runs example program
  @endcode

	To view the generated documentation, you can use the view_documentation make target:
	
  @code
  make view_documentation 
  @endcode 
	*
*/

/*! \page page_tutorial Tutorial: How to use HyPE
  \tableofcontents
  HyPE is organized as a library to allow easy integration in existing applications. You can choose between a dynamic and a static version of the library. Note that you have to link against the libraries HyPE uses, if you use the static version.
   * 
	* <b>ATTENTION: HyPE uses the thread library of boost for it's advanced features. There is a bug concerning applications compiled with the g++ compiler, because boost thread does not properly export all symbols until version 1.48. 
	* The Bug was fixed in <a href="http://www.boost.org/users/history/version_1_49_0.html">Boost 1.49</a>. 
	* The Workaound is to statically link against boost thread. Further details can be found <a href="https://svn.boost.org/trac/boost/ticket/2309">here</a>.</b>
	* 
	* To integrate HyPE in your project, you have to include the header file 
	*  @code
#include <stemod.hpp>
	*	@endcode
	*  and link against hype:
	*  @code
g++ -g  -Wl,-rpath,${PATH_TO_STEMOD}/lib -Wall -Werror -o <you application's name> <object files> -I${PATH_TO_STEMOD}/include -Bstatic -lboost_thread -pthread -Bdynamic -L${PATH_TO_STEMOD}/lib -lstemod -lboost_system -lboost_filesystem -lboost_program_options-mt -lloki -lrt
	*	@endcode
	
	 * \subsection sec_api_usage_hype  Use the API
	 * 
	 * The general concept of HyPE is to make decisions for your applications, which algorithm (processing device) should be used to perform an operation.
	 * Therefore, you first have to specify the Operations you wish to make decisions for and second you have to register your available algorithms for these operations.
	 * First we need a reference to the global Scheduler:
	 * @code
stemod::Scheduler& scheduler=stemod::Scheduler::instance();
	*	@endcode
	*  Now we can use the reference to interact with hype, e.g., adding an operation "SORT" and assigning to algortihms to the operation.
	 * @code
scheduler.addAlgorithm("SORT","CPU_Algorithm",stemod::CPU,"Least Squares 1D","Periodic Recomputation");
scheduler.addAlgorithm("SORT","GPU_Algorithm",stemod::GPU,"Least Squares 1D","Periodic Recomputation");
	*	@endcode
	 * The method addAlgorithm takes 5 parameters:
	 * -# the name of the operation ("SORT"), 
	 * -# the algorithm name ("CPU_Algorithm") that will be added to the operation ("SORT")
	 * -# A member of the enum stemod::ComputeDevice, which tells the HyPE the processing device, the algortihm is executed 
	 * 	- <b>ATTENTION: algorithm name has to be unique!</b> 
	 * -# the name of a statistical method ("Least Squares 1D")
	 * -# the name of a recomputation heuristic ("Periodic Recomputation")
	 * 
	 * After we assigned the operatiion "SORT" the algorithms "CPU_Algorithm" and "GPU_Algorithm", we are now ready to use the scheduling functionality of HyPE.
	 * First, we have to identify the parameters of a data set that have a high impact on the algorithms execution time. 
	 * In case of our sorting example, we identify the size of the input array as the most important feature value. Note that HyPE supports n feature values (n>=1).
	 * 
	 * To tell HyPE the feature value(s) of the data set that is to process, we have to store them in a stemod::Tuple object.
	 * @code
core::Tuple t;
t.push_back(Size_of_Input_Dataset);
	*	@endcode
	*
	 * Afterwards, we can ask HyPE, which algorithm we should use.
  	 * @code
stemod::SchedulingDecision sched_dec = scheduler.getOptimalAlgorithmName("SORT",t);
	*	@endcode
	 * We call the method getOptimalAlgorithmName on our scheduler reference with two parameters. The first is the oepration we want to perform and the second specifies the features of the input data set. (Note: In case of more than one feature values, you should always stick to one ordering, e.g., datasize, #distinct_keys. Changing this ordering at runtime will cause undefined behaviour.)
	 * The returned object sched_dec represents the decision of HyPE. We have to query it to identify the algorithm we should execute.
  	 * @code
if(sched_dec.getNameofChoosenAlgorithm()=="CPU_Algorithm"){
	//execute "CPU_Algorithm"
}else if(sched_dec.getNameofChoosenAlgorithm()=="GPU_Algorithm"){
	//execute "GPU_Algorithm"
}
	*	@endcode
		Note that you have to always execute the algorithm HyPE chooses, otherwise, all follwing calls to sched_dec.getNameofChoosenAlgorithm() will have undefined behavior.
	*  Since HyPE uses a feedback loop to refine the estimations of algorithm execution times, you have to measure the execution times of your algorithms and pass them back to HyPE.
	*  HyPE takes care of the nasty things and provides a high level interface for algorithm measurement.
	*  @code
AlgorithmMeasurement alg_measure(sched_dec); //has to be directly before algorithm execution
	//execute the choosen algortihm
alg_measure.afterAlgorithmExecution();  //has to be directly after algorithm termination
	*	@endcode
	 * The AlgorithmMeasurement object starts a timer and afterAlgorithmExecution() stops the timer. Note that the constructor of the AlgorithmMeasurement object needs a SchedulingDecision as parameter.
	 * When we put the handling of the SchedulingDecision together, we get the followign code sceleton:
	 * @code
if(sched_dec.getNameofChoosenAlgorithm()=="CPU_Algorithm"){
	AlgorithmMeasurement alg_measure(sched_dec);
		//execute "CPU_Algorithm"
	alg_measure.afterAlgorithmExecution(); 
}else if(sched_dec.getNameofChoosenAlgorithm()=="GPU_Algorithm"){
	AlgorithmMeasurement alg_measure(sched_dec);
		//execute "GPU_Algorithm"
	alg_measure.afterAlgorithmExecution(); 
}
	  * @endcode
	  
	   * The complete source code of this example can be found in the documentation \ref online_learning.cpp and in the examples directory of HyPE (examples/use_as_online_framework/online_learning.cpp).
		* 
		* 
 * */
 
/*! \page page_existing_plugins_hype Available Components
  \tableofcontents
  
  \section sec_statistical_method Statistical Methods
   
	 A Statistical Method learns the relation between the feature values of the input dataset and an algorithms execution time. It is a central part of HyPE, implementing the learning based execution time estiamtion. 
	  * Hence, it is crucial to select the appropriate statistical method depending on the algorithm and the application environment.
	  * Currently, HyPE supports one dimensional Least Square Method and Multi Linear Fitting.
	  * 
		\subsection subsec_statistical_method_leastsquares Least Square Method
		HyPE uses the least square solver of the ALGLIB Project. It is usually the candidate to choose, if an algorithm only depends on one input features, such as sorting.
		 
		\subsection subsec_statistical_method_multi_linear_fitting Multi Linear Fitting 
		HyPE uses the multi linear fitting functionality of the ALGLIB Project. You should choose Multi Linear Fitting, if an algorithm depends on multiple input features, such as selections (data size, selectivity).
		<b>Note that Multi Linear Fitting is currently limited to two features, but will support more in future versions of HyPE.</b>
  \section sec_recomputation_heuristic Recomputation Heuristics
   A Recomputation Heuristic implements the load adaption functionality of HyPE. If the load situation of a system dramatically changes, then it is very likely that the execution time of algorithm will change as well.
	 * To ensure sufficiently exact estimations, the learned approximation functions have to be updated. However, there is now 'perfect' point in time when to recompute the approximation functions. 
	 * Therefore, the user can select a Recomputation Heuristic, which is appropriate for the application.
	 \subsection subsec_recomputation_heuristic_periodic Periodic Recomputation
	  * The Periodic Recomputation Heuristic will recompute the approximation function of an algorithm after X executions of this algorithm. X is called Recomputation 
	  * Period and can be configured as well (see \ref page_configure_hype  for details). 
	  * You should use this Recomputation Heuristic if you want that HyPE refines its estimations at runtime to adapt at changing data, load, etc.
		 
	 \subsection subsec_recomputation_heuristic_oneshot Oneshot Recomputation
	  * The Oneshot Recomputation Heuristik will compute the approximation functions once after the initial training phase. You should choose this optimization criterion, if significant changes in the load in your system is seldom or have little impact on algorithms execution time.
	 
	 \subsection subsec_recomputation_heuristic_error_based Error based Recomputation (under development)

  \section sec_optimization_criterion Optimization Criterions
   An Optimization Criterion specifies what an "optimal" algorithm for your application is. Should it be the fastest? 
	Or would you like to select algorithms in a way that the throughput of your system is optimized? 
	 * Therefore, we implemented several strategies to make HyPE configurable and better usable in a broader application spectrum.   
	 \subsection subsec_optimization_criterion_response_time Response Time
	  The idea of Response Time optimization is to reduce the execution time of one operation by selecting the (estimated) fastest algorithm. 

	 \subsection subsec_optimization_criterion_throughput Throughput (experimental) 
*/



/*! \page page_configure_hype Configure HyPE
  \tableofcontents
  
   * HyPE can be configured in four ways. First, modify the Static_Configuration of HyPE, which sets default values for all variables. 
	* Second, update the configuration at runtime. The class Runtime_Configuration provides methods to change all modifiable variables. 
	* Note that not all variables are modifiable during runtime.
	* Third, create a configuration file 'stemod.conf', and add the variables with their corresponding values. 
	* Note, that the structure of the file for each line is variable_name=value and one line may at most contain one assignment.
	* Fourth, specify parameter values in environment variables.
	* 
	* -# modify the stemod::core::Static_Configuration of HyPE
	* -# update the configuration at runtime using stemod::core::Runtime_Configuration
	* -# create a configuration file 'hype.conf', and add the variables with their corresponding values
 	* 		- <b>help</b>                        produce help message
 	* 		- <b>length_of_trainingphase</b> set the number algorithms executions to complete training
 	* 		- <b>history_length</b> set the number of measurement pairs that are kept in the history (important for precision of approximation functions)
 	* 		- <b>recomputation_period</b> set the number of algorithm executions to trigger recomputation
 	* 		- <b>algorithm_maximal_idle_time</b> set the number of algorithm executions to trigger recomputation
 	* 		- <b>retraining_length</b> set the number of algorithm executions to trigger recomputation
	* 		- <b>ready_queue_length</b> set the number of operators that are queued on a processing device, before scheduling decision stops scheduling new operators (The idea is to wait how the done scheduling decisions turn out and to adjust the scheduling accordingly)
	* -# specify parameter values in environment variables:
	* 		- HYPE_LENGTH_OF_TRAININGPHASE
	* 		- HYPE_HISTORY_LENGTH
	* 		- HYPE_RECOMPUTATION_PERIOD
	*		- HYPE_ALGORITHM_MAXIMAL_IDLE_TIME
	* 		- HYPE_RETRAINING_LENGTH
	* 		- HYPE_READY_QUEUE_LENGTH
	* 
	* 
*/

/*! \page page_structure_hype Structure of HyPE
	* 
	* HyPE is structured to be a DBMS independent library, meaning templates and functions can be used without modifcation. 
	* However, to acheive this goal, the user has to implement a mapping layer, between HyPE and the hybrid DBMS:
	* 
	* \htmlonly
	* <p><img src="architecture.svg" align="center" width="40%"></p>
	* \endhtmlonly
	* 
	* 
	* 
	* 
	* 
*/

/*! \page page_extending_hype Extend HyPE
  \tableofcontents
  
   *   \section sec_statistical_method Statistical Methods
   *   Coming soon...
   *   \section sec_recomputation_heuristic Recomputation Heuristics
	*   Coming soon...
	*   \section sec_optimization_criterion Optimization Criterions
	*   Coming soon...
*/

/*! \page page_faq FAQ
   * 
	*  -# <b>What is HyPE?</b><br>
	*    HyPE is a Hybrid query Processing Engine build for automatic selection of processing units for coprocessing in database systems. The long-term goal of the project is to implement a fully fledged query processing engine, which is able to automatically generate and optimize a hybrid CPU/GPU physical query plan from a logical query plan.
	*  -# <b>When should I use HyPE?</b><br>
	*  You can use HyPE whenever you want to decide on a CPU and a GPU implementation of an operation in your application at run-time.
	*  -# <b>Under which license is HyPE distributed?</b><br>
	*  HyPE is licenced under the <a href="http://www.gnu.org/licenses/lgpl-3.0.txt">GNU LESSER GENERAL PUBLIC LICENSE - Version 3</a>. 
	*  -# <b>Which platforms are currently supported?</b><br>
	* 	HyPE compiles and runs under Linux and Windows (Cygwin). Native Windows support is planned for future releases.
	*  -# <b>I have a problem in using HyPE, how can I get help?</b><br>
	*  For information about the project, technical questions and bug reports: please contact the development team via <a href="http://wwwiti.cs.uni-magdeburg.de/~bress/">Sebastian Breß</a>.
	* 
*/
