
/** \example unittest_learning.cpp
 * This is an example of how to use the framework. This example is the unittest to test the learning procedure. To simple algorithms wait different times depending on their input data sizes. Note that one algorthm (CPU) is faster for little data sets and one is faster for big data sizes (GPU).
 */

#include <iostream>
#include <hype.hpp>

#include <boost/thread.hpp>

using namespace hype;
using namespace std;

void CPU_algorithm(size_t data_size){
	boost::this_thread::sleep(boost::posix_time::milliseconds(data_size ));

}

void GPU_algorithm(size_t data_size){
	boost::this_thread::sleep(boost::posix_time::milliseconds((data_size/2)+50 ));

}

int main(){

   	Scheduler& scheduler=Scheduler::instance();

	scheduler.addAlgorithm("SORT","CPU_Algorithm","Least Squares 1D","Periodic Recomputation");
	scheduler.addAlgorithm("SORT","GPU_Algorithm","Least Squares 1D","Periodic Recomputation");


	for(int i=0;i<100000;i++){
	Tuple t;
	t.push_back(rand()%300);
	uint64_t begin = core::getTimestamp();
	SchedulingDecision sched_dec = scheduler.getOptimalAlgorithmName("SORT",t);
	cout << "Line: " << i << " ";
	if(sched_dec.getNameofChoosenAlgorithm()=="CPU_Algorithm"){
		AlgorithmMeasurement alg_measure(sched_dec);
		CPU_algorithm(t[0]);
		alg_measure.afterAlgorithmExecution(); 
	}else if(sched_dec.getNameofChoosenAlgorithm()=="GPU_Algorithm"){
		AlgorithmMeasurement alg_measure(sched_dec);
		GPU_algorithm(t[0]);
		alg_measure.afterAlgorithmExecution(); 
	}
	
	uint64_t end = core::getTimestamp();

	uint64_t result = end-begin;
	if(begin>end) cout << "Fatal Error" << endl; 
	cout << result << endl;
	}

	
	//string algorithm_name = sched_dec.getNameofChoosenAlgorithm();

 return 0;
};

