
namespace hype{
	namespace core{
		
	class Static_Configuration{
		public:
		enum {
			length_of_initial_training_phase=10,
			length_of_history=1000,
			period_for_periodic_recomputation=100,
			maximal_time_where_algorithm_was_not_choosen=8,
			maximal_retraining_length=1,
			window_size_for_windowed_average_relative_estimation_error=100,
			maximal_slowdown_of_non_optimal_algorithm_in_percent=10, //%
			ready_queue_length_of_processing_devices=100
		};
		
	};
	
	class Runtime_Configuration{
	public:
		
		static Runtime_Configuration& instance() throw();
		
		bool setHistoryLength(unsigned int history_length) throw();
		
		bool setRecomputationPeriod(unsigned int length_of_recomputation_period) throw();
		
		bool setRetrainingLength(unsigned int retraining_length) throw();
		
		bool setAlgorithmMaximalIdleTime(unsigned int max_idle_time) throw();
		
		bool setMaximalReadyQueueLength(unsigned int max_ready_queue_length_of_processing_devices) throw();
		
		bool setOutlinerThreshold(double threshold) throw();
		
		unsigned int getTrainingLength() const throw();
		
		unsigned int getHistoryLength() const throw();
		
		unsigned int getRecomputationPeriod() const throw();
		
		unsigned int getRetrainingLength() const throw();
		
		unsigned int getAlgorithmMaximalIdleTime() const throw();
		
		unsigned int getRelativeErrorWindowSize() const throw();
		/*! \brief returns maximal slowdown in percent*/
		unsigned int getMaximalSlowdownOfNonOptimalAlgorithm() const throw();
		
		unsigned int getMaximalReadyQueueLength() const throw();
		
		double getOutlinerThreshold() const throw();
	private:
		Runtime_Configuration();
		Runtime_Configuration(const Runtime_Configuration&);
		Runtime_Configuration& operator=(const Runtime_Configuration&);
		
		unsigned int length_of_initial_training_phase_;
		unsigned int length_of_history_;
		unsigned int period_for_periodic_recomputation_;
		unsigned int maximal_time_where_algorithm_was_not_choosen_;
		unsigned int maximal_retraining_length_;
		unsigned int window_size_for_windowed_average_relative_estimation_error_;
		unsigned int maximal_slowdown_of_non_optimal_algorithm_in_percent_;
		unsigned int ready_queue_length_of_processing_devices_;
		//double outliner_threshold_in_percent_;
	};
	

		
	};//end namespace core
}; //end namespace stemod		
		