/***********************************************************************************************************
Copyright (c) 2012, Sebastian Breß, Otto-von-Guericke University of Magdeburg, Germany. All rights reserved.

This program and accompanying materials are made available under the terms of the 
GNU LESSER GENERAL PUBLIC LICENSE - Version 3, http://www.gnu.org/licenses/lgpl-3.0.txt
***********************************************************************************************************/

#pragma once

//C++ includes
#include <iostream>
#include <string>
#include <vector>
#include <fstream>
#include <sstream>
#include <limits>
#include <numeric>
//TR1 includes
#include <tr1/memory>
//C includes
#include <assert.h>
//#include <stdafx.h>
#include <math.h>

//loki library
#include <loki/Singleton.h>
#include <loki/Factory.h>

//boost includes
#include <boost/circular_buffer.hpp>
#include <boost/tokenizer.hpp>
#include <boost/filesystem.hpp>
#include <boost/lexical_cast.hpp>

//STEMOD includes
#include <core/time_measurement.hpp>
#include <core/measurementpair.hpp>

//Alglib
//#include <interpolation.h>



namespace hype{
	namespace core{

		//stores measurement pairs as columns, not as rows
		class MeasurementPairRingbuffer{
		public:
		MeasurementPairRingbuffer();
	
		MeasurementPairRingbuffer(size_t size);

		unsigned int size() const throw();
		
		bool store(std::ostream &out) const;

	   void set_maximal_number_of_measurement_pairs(size_t size);

		bool addMeasurementPair(const MeasurementPair& mp);

		//not const functions, since ring buffer has to be linearized to return column
		const std::vector<EstimatedTime> getColumnEstimations();

		const std::vector<MeasuredTime> getColumnMeasurements();

		const std::vector<Tuple> getColumnFeatureValues();
		
		void clear() throw();
		
		private:
 		  boost::circular_buffer<Tuple> feature_values_;//(2000);
 		  boost::circular_buffer<MeasuredTime> measured_times_;//(2000);
   	  boost::circular_buffer<EstimatedTime> estimated_times_;//(2000);	

		};

	}; //end namespace core
}; //end namespace stemod


