/***********************************************************************************************************
Copyright (c) 2012, Sebastian Breß, Otto-von-Guericke University of Magdeburg, Germany. All rights reserved.

This program and accompanying materials are made available under the terms of the 
GNU LESSER GENERAL PUBLIC LICENSE - Version 3, http://www.gnu.org/licenses/lgpl-3.0.txt
***********************************************************************************************************/

#pragma once

#include <iostream>
#include <fstream>
#include <string>
#include <vector>
#include <numeric>

#include <config/global_definitions.hpp>

#include <core/time_measurement.hpp>
#include <core/measurementpair.hpp>



namespace hype{
	namespace core{



class Offline_Algorithm{
	public:			
	const std::string& getAlgorithmName() const throw();
	
	const std::string& getOperationName() const throw();
	
	ComputeDevice getComputeDevice() const throw();

	unsigned int getNumberOfMeasurementPairs() const throw();

	Offline_Algorithm(ComputeDevice device,std::string algorithm_name, std::string opname, std::string filepath);

	core::MeasurementPair getNext();
	
	bool hasNext();
	
	void reset();
	
	//the first line of a file determines, how many feature values are expected: for n values, the first n-1 values are considered feature Values and the remaining 1 value is considered the measured execution time
	void loadMeasurementpairsfromFile(std::string filepath);

	void storeInFile(const std::string& file_name);

	void printstoredMeasurementpairs();

	static std::vector<Offline_Algorithm> randomize_dataset_of_offline_algorithms(std::vector<Offline_Algorithm> offline_algorithms);

	private:
	std::vector<core::MeasurementPair> offline_mesurement_pairs_;
	ComputeDevice device_;
	std::string name_;
	std::string operation_name_;
	unsigned int current_mesurementpair_index_;
	std::string filepath_;
};

	}; //end namespace core
}; //end namespace stemod

