/***********************************************************************************************************
Copyright (c) 2012, Sebastian Breß, Otto-von-Guericke University of Magdeburg, Germany. All rights reserved.

This program and accompanying materials are made available under the terms of the 
GNU LESSER GENERAL PUBLIC LICENSE - Version 3, http://www.gnu.org/licenses/lgpl-3.0.txt
***********************************************************************************************************/
#pragma once

#include <iostream>
#include <string>
#include <vector>
#include <map>
#include <tr1/memory>

//loki library
#include <loki/Singleton.h>
#include <loki/Factory.h>

//STEMOD includes
#include <core/time_measurement.hpp>


namespace hype{
	namespace core{

class Algorithm; //forward declaration


class StatisticalMethod {
/*! \todo move functionality of keeping a approximation function computed bit from derived classes in this base class*/

public:
	virtual const EstimatedTime computeEstimation(const Tuple& input_values) = 0;

	virtual bool recomuteApproximationFunction(Algorithm& algorithm) = 0;

	virtual bool inTrainingPhase() const throw() = 0;

	virtual void retrain() = 0;

	virtual ~StatisticalMethod(){}

protected:
	StatisticalMethod(const std::string& name);

private:
	std::string name_;
	
};

//factory function
const std::tr1::shared_ptr<StatisticalMethod> getNewStatisticalMethodbyName(std::string name);

typedef Loki::Factory< StatisticalMethod, std::string> StatisticalMethodFactory; //aFactory;
//typedef Loki::Singleton<StatisticalMethodFactory> StatisticalMethodFactorySingleton;

class StatisticalMethodFactorySingleton{

	public:

	static StatisticalMethodFactory& Instance(){		
		static StatisticalMethodFactory factory;
		return factory;
	}

};


	typedef std::tr1::shared_ptr<StatisticalMethod> StatisticalMethodPtr;
	typedef std::map<std::string,StatisticalMethodPtr> StatisticalMethodMap;

	}; //end namespace core
}; //end namespace stemod


