
#pragma once

//#include <core/scheduling_decision.hpp>

//#include <boost/shared_ptr.hpp>

#include <stack>

#include <query_processing/node.hpp>
#include <query_processing/physical_query_plan.hpp>

namespace hype
{
	namespace queryprocessing
	{

		struct print_NodePtr_functor {
			void operator()(NodePtr ptr) const {
				for(unsigned int i=0; i<ptr->getLevel(); i++)
					std::cout << "\t";
				std::cout << "Operation: " << ptr->getOperationName() << std::endl;
			}
		};

		struct assign_treelevel_NodePtr_functor {
			void operator()(NodePtr ptr) {
				if(ptr.get()!=NULL) {
					NodePtr left=ptr->getLeft();
					NodePtr right=ptr->getRight();
					if(left.get()!=NULL) {
						left->setLevel(ptr->getLevel()+1);
					}
					if(right.get()!=NULL) {
						right->setLevel(ptr->getLevel()+1);
					}


				}
			}
		};

		template <typename Type>
		struct Construct_Physical_Query_Plan_functor {
			typedef typename OperatorMapper_Helper_Template<Type>::TypedOperatorPtr TypedOperatorPtr;
			typedef typename OperatorMapper_Helper_Template<Type>::TypedNodePtr TypedNodePtr;

			typedef std::map<TypedNodePtr,TypedOperatorPtr> Map;

			Construct_Physical_Query_Plan_functor() : root_(),map_log_op_to_phy_op_() {

			}

			TypedOperatorPtr getRootOperator() {
				return root_;
			}

			void operator()(TypedNodePtr ptr) {
				std::cout << typeid(ptr).name();
				
				//ptr->getOptimalOperator();

				if(ptr) {

					TypedOperatorPtr left_op;
					TypedOperatorPtr right_op;

					TypedNodePtr left=boost::static_pointer_cast<typename TypedNodePtr::element_type>(ptr->getLeft());
					TypedNodePtr right=boost::static_pointer_cast<typename TypedNodePtr::element_type>(ptr->getRight());
					if(left!=NULL) {
						typename Map::iterator it=map_log_op_to_phy_op_.find(left);
						if(it!=map_log_op_to_phy_op_.end())
							left_op=it->second;
					}
					if(right!=NULL) {
						assert(left!=0); //check whether convention is obeyed
						typename Map::iterator it=map_log_op_to_phy_op_.find(right);
						if(it!=map_log_op_to_phy_op_.end())
							right_op=it->second;
					}

					TypedOperatorPtr phy_op_ptr = ptr->getOptimalOperator(left_op,right_op);
					map_log_op_to_phy_op_[ptr]=phy_op_ptr;
					root_=phy_op_ptr; //update root (only important thing that the correct root node is stored in here at the end, when get funcion is called)

					
				}
				
			}
			
			TypedOperatorPtr root_;
			//maps logical to physical operator
			Map map_log_op_to_phy_op_;
		};


		template <typename Type>
		class LogicalQueryPlan
		{
			public:
				typedef Type NodeElementType;
				typedef typename OperatorMapper_Helper_Template<Type>::Physical_Operator_Map Physical_Operator_Map;
				typedef typename OperatorMapper_Helper_Template<Type>::TypedOperatorPtr TypedOperatorPtr;
				typedef typename OperatorMapper_Helper_Template<Type>::TypedNodePtr TypedNodePtr;
				typedef typename OperatorMapper_Helper_Template<Type>::PhysicalQueryPlanPtr PhysicalQueryPlanPtr;

				LogicalQueryPlan(TypedNodePtr root) : root_(root) {
					root_->setLevel(0);
					//set the tree level of each node appropriately
					traverse(assign_treelevel_NodePtr_functor());
				}

				const PhysicalQueryPlanPtr convertToPhysicalQueryPlan() {
					/*
					std::list<TypedNodePtr> operator_list = getLevelOrder();
					operator_list.reverse();
					typename std::list<TypedNodePtr>::iterator it;



					for(it=operator_list.begin(); it!=operator_list.end(); ++it) {
						it->getOptimalOperator();
					}*/

					Construct_Physical_Query_Plan_functor<Type> functor = reverse_traverse(Construct_Physical_Query_Plan_functor<Type>());

					TypedOperatorPtr root =functor.getRootOperator();
					PhysicalQueryPlanPtr ptr(new PhysicalQueryPlan<Type>(root));

					return ptr;
				}

				template<class UnaryFunction>
				UnaryFunction traverse(UnaryFunction f) {

					std::list<TypedNodePtr> queue;

					if(root_.get()==NULL) return f;

					queue.push_back(root_);

					while(!queue.empty()) {
						TypedNodePtr currentNode=queue.front();
						queue.pop_front(); //delete first element

						if(currentNode.get()!=NULL) {
							TypedNodePtr left=boost::static_pointer_cast<typename TypedNodePtr::element_type>(currentNode->getLeft());
							TypedNodePtr right=boost::static_pointer_cast<typename TypedNodePtr::element_type>(currentNode->getRight());

							f(currentNode);

							if(left.get()!=NULL) {
								queue.push_back(left);
							}
							if(right.get()!=NULL) {
								queue.push_back(right);
							}

						}

					}

					return f;
				}

				template<class UnaryFunction>
				UnaryFunction reverse_traverse(UnaryFunction f) {

					std::list<TypedNodePtr> queue = this->getLevelOrder();
					queue.reverse();
					//process in reverse level order
					typename std::list<TypedNodePtr>::iterator it;
					for (it=queue.begin();it!=queue.end();++it){
						f(*it);
					}
					
					/*
					std::list<TypedNodePtr> queue;
					std::list<TypedNodePtr> result_queue;
					
					if(root_.get()==NULL) return f;

					queue.push_back(root_);

					while(!queue.empty()) {
						TypedNodePtr currentNode=queue.front();
						queue.pop_front(); //delete frist element

						if(currentNode.get()!=NULL) {
							TypedNodePtr left=currentNode->getLeft();
							TypedNodePtr right=currentNode->getRight();

							f(currentNode);

							if(left.get()!=NULL) {
								queue.push_back(left);
							}
							if(right.get()!=NULL) {
								queue.push_back(right);
							}

						}

					}
					*/
					return f;
				}

				std::list<TypedNodePtr> getLevelOrder() {
					std::list<TypedNodePtr> queue;
					std::list<TypedNodePtr> result_queue;

					if(root_.get()==NULL) return result_queue;

					queue.push_back(root_);
					result_queue.push_back(root_);

					while(!queue.empty()) {
						TypedNodePtr currentNode=queue.front();
						queue.pop_front(); //delete first element

						if(currentNode.get()!=NULL) {
							TypedNodePtr left=boost::static_pointer_cast<typename TypedNodePtr::element_type>(currentNode->getLeft());
							TypedNodePtr right=boost::static_pointer_cast<typename TypedNodePtr::element_type>(currentNode->getRight());
							//f(currentNode);

							if(left.get()!=NULL) {
								queue.push_back(left);
								result_queue.push_back(left);
							}
							if(right.get()!=NULL) {
								queue.push_back(right);
								result_queue.push_back(right);
							}

						}

					}
					return result_queue;
				}

				template<class UnaryFunction>
				UnaryFunction traverse_preorder(UnaryFunction f) {
					std::stack<TypedNodePtr> nodeStack;
					TypedNodePtr curr = root_;
					while (true) {
						if (curr.get()!=NULL) {

							f(curr);

							nodeStack.push(curr);
							curr = boost::static_pointer_cast<typename TypedNodePtr::element_type>(curr->getLeft());
							continue;
						}
						if (!nodeStack.size()) {
							return f;
						}
						curr = nodeStack.top();
						nodeStack.pop();
						curr = boost::static_pointer_cast<typename TypedNodePtr::element_type>(curr->getRight());
					}
					return f;
				}


				void print() {

					traverse_preorder(print_NodePtr_functor());

				}

			private:
				TypedNodePtr root_;

		};

	}; //end namespace queryprocessing
}; //end namespace stemod
